﻿using System;
using EMS.Math.Equ;

namespace Equ.Net_Demo
{
    class Program
    {
        //User defined function
        static double pow3(double x)
        {
            return x * x * x;
        }

       unsafe static void Main(string[] args)
        {
            string mathExp = "1.2e-3+x+2.3*(1-sin(y/3.14)^2)-z";
            string varList = "x,y,z";
            double[] data = new double[] { 1, 2.5, 1.7e-2 };
            double res = default(double);

            //Initialize EquTranslator algorithm first
            Translator equ = new Translator();

            try
            {
                //****************************************************
                //Evaluate math expression for a given point "data"
                res = equ.Evaluate(mathExp, varList, data);


                data[0] = 2.3; data[1] = -5;
                res = equ.Evaluate("x/y+(x^2-y^2)", "x,y", data);

                //****************************************************
                //Add user's function to EquTranslator
                equ.AddFunction("p3", pow3);

                //Calculate a set of points for one math expression
                //Build it first
                equ.Build("3.14*p3(x)+sin(y)-sqrt(z+x)", "x,y,z");

                res = 0;
                for (int i = 0; i < 1000; i++)
                {
                    data[0] = data[0] + 1;
                    data[1] = i + 1;
                    data[2] = data[0] / data[1];
                    //Do calculation for a given point
                    res += equ.CalcFor(data);
                }


                //****************************************************
                //Memory for data must be pinned in order to use 
                //"Compile" and "Run" functions.
                double* pinData = stackalloc double[3];
               
                equ.Compile(mathExp, varList, pinData);
                res = 0;
                for (int i = 0; i < 1000; i++)
                {
                    pinData[0] = pinData[0] + 1;
                    pinData[1] = i + 1;
                    pinData[2] = pinData[0] / data[1];
                    //Do calculation for a given point
                    res += equ.Run();
                }

                //****************************************************
                //Negative test
                equ.RemoveFunction("p3");
                res = equ.Evaluate("3.14*p3(x)+sin(y)-sqrt(z+x)", "x,y,z", data);
            }
            catch (TranslatorException ex)
            {
                Console.WriteLine(ex.m_errCode);
                Console.WriteLine(ex.Message);
                Console.WriteLine(ex.m_errPosition);
            }

            return;
        }
    }
}

